@@ -141,7 +141,7 @@ ARRAY_BIND_THRESHOLD = 512
 
 
 class Analyzer:
-    def __init__(self, session: "snowflake.snowpark.session.Session"):
+    def __init__(self, session: "snowflake.snowpark.session.Session") -> None:
         self.session = session
         self.plan_builder = SnowflakePlanBuilder(self.session)
         self.generated_alias_maps = {}
@@ -552,8 +552,9 @@ class Analyzer:
         if isinstance(logical_plan, SnowflakeCreateTable):
             return self.plan_builder.save_as_table(
                 logical_plan.table_name,
+                logical_plan.column_names,
                 logical_plan.mode,
-                logical_plan.create_temp_table,
+                logical_plan.table_type,
                 resolved_children[logical_plan.children[0]],
             )
 
