@@ -2,7 +2,7 @@
 # Copyright (c) 2012-2022 Snowflake Computing Inc. All rights reserved.
 #
 
-from typing import Dict, List, Optional
+from typing import Dict, Iterable, List, Optional
 
 from snowflake.snowpark._internal.analyzer.expression import Expression
 from snowflake.snowpark._internal.analyzer.snowflake_plan_node import LogicalPlan
@@ -15,7 +15,7 @@ class TableFunctionPartitionSpecDefinition(Expression):
         over: bool = False,
         partition_spec: Optional[List[Expression]] = None,
         order_spec: Optional[List[SortOrder]] = None,
-    ):
+    ) -> None:
         super().__init__()
         self.over = over
         self.partition_spec = partition_spec
@@ -27,16 +27,18 @@ class TableFunctionExpression(Expression):
         self,
         func_name: str,
         partition_spec: Optional[TableFunctionPartitionSpecDefinition] = None,
-    ):
+        aliases: Optional[Iterable[str]] = None,
+    ) -> None:
         super().__init__()
         self.func_name = func_name
         self.partition_spec = partition_spec
+        self.aliases = aliases
 
 
 class FlattenFunction(TableFunctionExpression):
     def __init__(
         self, input: Expression, path: str, outer: bool, recursive: bool, mode: str
-    ):
+    ) -> None:
         super().__init__("flatten")
         self.input = input
         self.path = path
@@ -51,7 +53,7 @@ class PosArgumentsTableFunction(TableFunctionExpression):
         func_name: str,
         args: List[Expression],
         partition_spec: Optional[TableFunctionPartitionSpecDefinition] = None,
-    ):
+    ) -> None:
         super().__init__(func_name, partition_spec)
         self.args = args
 
@@ -62,26 +64,30 @@ class NamedArgumentsTableFunction(TableFunctionExpression):
         func_name: str,
         args: Dict[str, Expression],
         partition_spec: Optional[TableFunctionPartitionSpecDefinition] = None,
-    ):
+    ) -> None:
         super().__init__(func_name, partition_spec)
         self.args = args
 
 
 class TableFunctionRelation(LogicalPlan):
-    def __init__(self, table_function: TableFunctionExpression):
+    def __init__(self, table_function: TableFunctionExpression) -> None:
         super().__init__()
         self.table_function = table_function
 
 
 class TableFunctionJoin(LogicalPlan):
-    def __init__(self, child: LogicalPlan, table_function: TableFunctionExpression):
+    def __init__(
+        self, child: LogicalPlan, table_function: TableFunctionExpression
+    ) -> None:
         super().__init__()
         self.children = [child]
         self.table_function = table_function
 
 
 class Lateral(LogicalPlan):
-    def __init__(self, child: LogicalPlan, table_function: TableFunctionExpression):
+    def __init__(
+        self, child: LogicalPlan, table_function: TableFunctionExpression
+    ) -> None:
         super().__init__()
         self.children = [child]
         self.table_function = table_function
