@@ -4,14 +4,16 @@
 #
 import functools
 import os
+import sys
 import time
 from logging import getLogger
-from typing import IO, Any, Dict, Iterator, List, Optional, Union
+from typing import IO, Any, Dict, Iterator, List, Optional, Set, Tuple, Union
 
 import snowflake.connector
 from snowflake.connector import SnowflakeConnection, connect
+from snowflake.connector.constants import FIELD_ID_TO_NAME
 from snowflake.connector.cursor import ResultMetadata, SnowflakeCursor
-from snowflake.connector.errors import NotSupportedError
+from snowflake.connector.errors import NotSupportedError, ProgrammingError
 from snowflake.connector.network import ReauthenticationRequest
 from snowflake.connector.options import pandas
 from snowflake.snowpark._internal.analyzer.analyzer_utils import (
@@ -19,6 +21,7 @@ from snowflake.snowpark._internal.analyzer.analyzer_utils import (
     quote_name,
     quote_name_without_upper_casing,
 )
+from snowflake.snowpark._internal.analyzer.datatype_mapper import str_to_sql
 from snowflake.snowpark._internal.analyzer.expression import Attribute
 from snowflake.snowpark._internal.analyzer.schema_utils import (
     convert_result_meta_to_attribute,
@@ -98,7 +101,6 @@ class ServerConnection:
                         ex.cause
                     )
                 except Exception as ex:
-                    # TODO: SNOW-363951 handle telemetry
                     raise ex
 
             return wrap
@@ -120,6 +122,7 @@ class ServerConnection:
                             func.__name__, duration, sfqid
                         )
                     logger.debug(f"Finished in {duration:.4f} secs")
+                    return result
 
                 return wrap
 
@@ -129,13 +132,15 @@ class ServerConnection:
         self,
         options: Dict[str, Union[int, str]],
         conn: Optional[SnowflakeConnection] = None,
-    ):
+    ) -> None:
         self._lower_case_parameters = {k.lower(): v for k, v in options.items()}
         self._add_application_name()
         self._conn = conn if conn else connect(**self._lower_case_parameters)
+        if "password" in self._lower_case_parameters:
+            self._lower_case_parameters["password"] = None
         self._cursor = self._conn.cursor()
         self._telemetry_client = TelemetryClient(self._conn)
-        self._query_listener = set()  # type: set[QueryHistory]
+        self._query_listener: Set[QueryHistory] = set()
         # The session in this case refers to a Snowflake session, not a
         # Snowpark session
         self._telemetry_client.send_session_created_telemetry(not bool(conn))
@@ -184,18 +189,12 @@ class ServerConnection:
         )
 
     @_Decorator.wrap_exception
-    def _get_current_parameter(
-        self, param: str, unquoted: bool = False
-    ) -> Optional[str]:
+    def _get_current_parameter(self, param: str, quoted: bool = True) -> Optional[str]:
         name = getattr(self._conn, param) or self._get_string_datum(
             f"SELECT CURRENT_{param.upper()}()"
         )
         return (
-            (
-                quote_name_without_upper_casing(name)
-                if not unquoted
-                else escape_quotes(name)
-            )
+            (quote_name_without_upper_casing(name) if quoted else escape_quotes(name))
             if name
             else None
         )
@@ -204,7 +203,7 @@ class ServerConnection:
         rows = result_set_to_rows(self.run_query(query)["data"])
         return rows[0][0] if len(rows) > 0 else None
 
-    @_Decorator.wrap_exception
+    @SnowflakePlan.Decorator.wrap_exception
     def get_result_attributes(self, query: str) -> List[Attribute]:
         return convert_result_meta_to_attribute(self._cursor.describe(query))
 
@@ -222,8 +221,17 @@ class ServerConnection:
         if is_in_stored_procedure():
             file_name = os.path.basename(path)
             target_path = _build_target_path(stage_location, dest_prefix)
-            # upload_stream directly consume stage path, so we don't need to normalize it
-            self._cursor.upload_stream(open(path, "rb"), f"{target_path}/{file_name}")
+            try:
+                # upload_stream directly consume stage path, so we don't need to normalize it
+                self._cursor.upload_stream(
+                    open(path, "rb"), f"{target_path}/{file_name}"
+                )
+            except ProgrammingError as pe:
+                tb = sys.exc_info()[2]
+                ne = SnowparkClientExceptionMessages.SQL_EXCEPTION_FROM_PROGRAMMING_ERROR(
+                    pe
+                )
+                raise ne.with_traceback(tb) from None
         else:
             uri = normalize_local_file(path)
             return self.run_query(
@@ -249,16 +257,24 @@ class ServerConnection:
         compress_data: bool = True,
         source_compression: str = "AUTO_DETECT",
         overwrite: bool = False,
+        is_in_udf: bool = False,
     ) -> Optional[Dict[str, Any]]:
         uri = normalize_local_file(f"/tmp/placeholder/{dest_filename}")
         try:
             if is_in_stored_procedure():
                 input_stream.seek(0)
                 target_path = _build_target_path(stage_location, dest_prefix)
-                # upload_stream directly consume stage path, so we don't need to normalize it
-                self._cursor.upload_stream(
-                    input_stream, f"{target_path}/{dest_filename}"
-                )
+                try:
+                    # upload_stream directly consume stage path, so we don't need to normalize it
+                    self._cursor.upload_stream(
+                        input_stream, f"{target_path}/{dest_filename}"
+                    )
+                except ProgrammingError as pe:
+                    tb = sys.exc_info()[2]
+                    ne = SnowparkClientExceptionMessages.SQL_EXCEPTION_FROM_PROGRAMMING_ERROR(
+                        pe
+                    )
+                    raise ne.with_traceback(tb) from None
             else:
                 return self.run_query(
                     _build_put_statement(
@@ -276,9 +292,14 @@ class ServerConnection:
         # https://docs.python.org/3/library/io.html#io.IOBase.close
         except ValueError as ex:
             if input_stream.closed:
-                raise SnowparkClientExceptionMessages.SERVER_UDF_UPLOAD_FILE_STREAM_CLOSED(
-                    dest_filename
-                )
+                if is_in_udf:
+                    raise SnowparkClientExceptionMessages.SERVER_UDF_UPLOAD_FILE_STREAM_CLOSED(
+                        dest_filename
+                    )
+                else:
+                    raise SnowparkClientExceptionMessages.SERVER_UPLOAD_FILE_STREAM_CLOSED(
+                        dest_filename
+                    )
             else:
                 raise ex
 
@@ -307,7 +328,8 @@ class ServerConnection:
             )
             logger.debug(f"Execute query [queryID: {results_cursor.sfqid}] {query}")
         except Exception as ex:
-            logger.error(f"Failed to execute query {query}\n{ex}")
+            query_id_log = f" [queryID: {ex.sfqid}]" if hasattr(ex, "sfqid") else ""
+            logger.error(f"Failed to execute query{query_id_log} {query}\n{ex}")
             raise ex
 
         # fetch_pandas_all/batches() only works for SELECT statements
@@ -355,12 +377,12 @@ class ServerConnection:
             plan, to_pandas, to_iter, **kwargs
         )
         if to_pandas:
-            return result_set
+            return result_set["data"]
         else:
             if to_iter:
-                return result_set_to_iter(result_set, result_meta)
+                return result_set_to_iter(result_set["data"], result_meta)
             else:
-                return result_set_to_rows(result_set, result_meta)
+                return result_set_to_rows(result_set["data"], result_meta)
 
     @SnowflakePlan.Decorator.wrap_exception
     def get_result_set(
@@ -369,12 +391,19 @@ class ServerConnection:
         to_pandas: bool = False,
         to_iter: bool = False,
         **kwargs,
-    ) -> (
-        Union[
-            List[Any], "pandas.DataFrame", SnowflakeCursor, Iterator["pandas.DataFrame"]
+    ) -> Tuple[
+        Dict[
+            str,
+            Union[
+                List[Any],
+                "pandas.DataFrame",
+                SnowflakeCursor,
+                Iterator["pandas.DataFrame"],
+                str,
+            ],
         ],
         List[ResultMetadata],
-    ):
+    ]:
         action_id = plan.session._generate_new_action_id()
 
         result, result_meta = None, None
@@ -410,31 +439,37 @@ class ServerConnection:
         if result is None:
             raise SnowparkClientExceptionMessages.SQL_LAST_QUERY_RETURN_RESULTSET()
 
-        return result["data"], result_meta
+        return result, result_meta
 
     def get_result_and_metadata(
         self, plan: SnowflakePlan, **kwargs
-    ) -> (List[Row], List[Attribute]):
+    ) -> Tuple[List[Row], List[Attribute]]:
         result_set, result_meta = self.get_result_set(plan, **kwargs)
-        result = result_set_to_rows(result_set)
+        result = result_set_to_rows(result_set["data"])
         meta = convert_result_meta_to_attribute(result_meta)
         return result, meta
 
+    def get_result_query_id(self, plan: SnowflakePlan, **kwargs) -> str:
+        # get the iterator such that the data is not fetched
+        result_set, _ = self.get_result_set(plan, to_iter=True, **kwargs)
+        return result_set["sfqid"]
+
     @_Decorator.wrap_exception
     def run_batch_insert(self, query: str, rows: List[Row], **kwargs) -> None:
         # with qmark, Python data type will be dynamically mapped to Snowflake data type
         # https://docs.snowflake.com/en/user-guide/python-connector-api.html#data-type-mappings-for-qmark-and-numeric-bindings
         params = [list(row) for row in rows]
+        statement_params = kwargs.get("_statement_params")
         query_tag = (
-            kwargs["_statement_params"]["QUERY_TAG"]
-            if "_statement_params" in kwargs
-            and "QUERY_TAG" in kwargs["_statement_params"]
+            statement_params["QUERY_TAG"]
+            if statement_params is not None
+            and "QUERY_TAG" in statement_params
             and not is_in_stored_procedure()
             else None
         )
         if query_tag:
             set_query_tag_cursor = self._cursor.execute(
-                f"alter session set query_tag='{query_tag}'"
+                f"alter session set query_tag = {str_to_sql(query_tag)}"
             )
             self.notify_query_listeners(
                 QueryRecord(set_query_tag_cursor.sfqid, set_query_tag_cursor.query)
@@ -450,17 +485,15 @@ class ServerConnection:
             self.notify_query_listeners(
                 QueryRecord(unset_query_tag_cursor.sfqid, unset_query_tag_cursor.query)
             )
-        logger.debug(f"Execute batch insertion query %s", query)
+        logger.debug("Execute batch insertion query %s", query)
 
     def _fix_pandas_df_integer(self, pd_df: "pandas.DataFrame") -> "pandas.DataFrame":
-        """To fix https://snowflakecomputing.atlassian.net/browse/SNOW-562208
-        TODO: remove this after Python connector does the conversion: https://snowflakecomputing.atlassian.net/browse/SNOW-562586
-        """
         for column_metadata, pandas_dtype, pandas_col_name in zip(
             self._cursor.description, pd_df.dtypes, pd_df.columns
         ):
             if (
-                column_metadata.precision is not None
+                FIELD_ID_TO_NAME.get(column_metadata.type_code) == "FIXED"
+                and column_metadata.precision is not None
                 and column_metadata.scale == 0
                 and not str(pandas_dtype).startswith("int")
             ):
