@@ -10,6 +10,7 @@ from logging import getLogger
 from types import ModuleType
 from typing import (
     Callable,
+    Dict,
     Iterable,
     List,
     NamedTuple,
@@ -22,6 +23,7 @@ from typing import (
 import cloudpickle
 
 import snowflake.snowpark
+from snowflake.snowpark._internal import code_generation
 from snowflake.snowpark._internal.type_utils import (
     convert_sp_to_sf_type,
     python_type_to_snow_type,
@@ -38,7 +40,12 @@ from snowflake.snowpark._internal.utils import (
     unwrap_stage_location_single_quote,
     validate_object_name,
 )
-from snowflake.snowpark.types import DataType, PandasDataFrameType, PandasSeriesType
+from snowflake.snowpark.types import (
+    DataType,
+    PandasDataFrameType,
+    PandasSeriesType,
+    StructType,
+)
 
 logger = getLogger(__name__)
 
@@ -50,6 +57,9 @@ _DEFAULT_HANDLER_NAME = "compute"
 # because zip compression ratio is quite high.
 _MAX_INLINE_CLOSURE_SIZE_BYTES = 8192
 
+# Every table function handler class must define the process method.
+TABLE_FUNCTION_PROCESS_METHOD = "process"
+
 
 class UDFColumn(NamedTuple):
     datatype: DataType
@@ -68,19 +78,34 @@ def get_types_from_type_hints(
         # For Python 3.10+, the result values of get_type_hints()
         # will become strings, which we have to change the implementation
         # here at that time. https://www.python.org/dev/peps/pep-0563/
-        python_types_dict = get_type_hints(func)
-    else:
-        python_types_dict = (
-            retrieve_func_type_hints_from_source(func[0], func[1])
-            if is_local_python_file(func[0])
-            else {}
+        python_types_dict = get_type_hints(
+            getattr(func, TABLE_FUNCTION_PROCESS_METHOD, func)
         )
+    else:
+        if object_type == TempObjectType.TABLE_FUNCTION:
+            python_types_dict = (
+                retrieve_func_type_hints_from_source(
+                    func[0], TABLE_FUNCTION_PROCESS_METHOD, class_name=func[1]
+                )  # use method process of a UDTF handler class.
+                if is_local_python_file(func[0])
+                else {}
+            )
+        else:
+            python_types_dict = (
+                retrieve_func_type_hints_from_source(func[0], func[1])
+                if is_local_python_file(func[0])
+                else {}
+            )
 
-    return_type = (
-        python_type_to_snow_type(python_types_dict["return"])[0]
-        if "return" in python_types_dict
-        else None
-    )
+    if object_type == TempObjectType.TABLE_FUNCTION:
+        return_type = None
+        # The return type is processed in udtf.py. Return None here.
+    else:
+        return_type = (
+            python_type_to_snow_type(python_types_dict["return"])[0]
+            if "return" in python_types_dict
+            else None
+        )
     input_types = []
 
     # types are in order
@@ -107,6 +132,8 @@ def get_error_message_abbr(object_type: TempObjectType) -> str:
         return "udf"
     if object_type == TempObjectType.PROCEDURE:
         return "stored proc"
+    if object_type == TempObjectType.TABLE_FUNCTION:
+        return "table function"
     raise ValueError(f"Expect FUNCTION of PROCEDURE, but get {object_type}")
 
 
@@ -295,7 +322,7 @@ def cleanup_failed_permanent_registration(
                 "Finished removing Snowpark uploaded file: %s",
                 upload_file_stage_location,
             )
-        except BaseException as clean_ex:
+        except Exception as clean_ex:
             logger.warning("Failed to clean uploaded file: %s", clean_ex)
 
 
@@ -321,14 +348,19 @@ def pickle_function(func: Callable) -> bytes:
 def generate_python_code(
     func: Callable,
     arg_names: List[str],
+    object_type: TempObjectType,
     is_pandas_udf: bool,
     is_dataframe_input: bool,
     max_batch_size: Optional[int] = None,
+    source_code_display: bool = False,
 ) -> str:
     # if func is a method object, we need to extract the target function first to check
     # annotations. However, we still serialize the original method because the extracted
     # function will have an extra argument `cls` or `self` from the class.
-    target_func = getattr(func, "__func__", func)
+    if object_type == TempObjectType.TABLE_FUNCTION:
+        target_func = getattr(func, TABLE_FUNCTION_PROCESS_METHOD)
+    else:
+        target_func = getattr(func, "__func__", func)
 
     # clear the annotations because when the user annotates Variant and Geography,
     # which are from snowpark modules and will not work on the server side
@@ -346,41 +378,110 @@ def generate_python_code(
         pickled_func = pickle_function(func)
     args = ",".join(arg_names)
 
+    try:
+        source_code_comment = (
+            code_generation.generate_source_code(func) if source_code_display else ""
+        )
+    except Exception as exc:
+        error_msg = (
+            f"Source code comment could not be generated for {func} due to error {exc}."
+        )
+        logger.debug(error_msg)
+        # We shall also have telemetry for the code generation
+        # check https://snowflakecomputing.atlassian.net/browse/SNOW-651381
+        source_code_comment = code_generation.comment_source_code(error_msg)
+
     deserialization_code = f"""
 import pickle
 
 func = pickle.loads(bytes.fromhex('{pickled_func.hex()}'))
+{source_code_comment}
 """.rstrip()
-    if is_pandas_udf:
-        pandas_code = f"""
+
+    if object_type == TempObjectType.PROCEDURE:
+        func_code = f"""
+def {_DEFAULT_HANDLER_NAME}({args}):
+    return func({args})
+"""
+    else:
+        func_code = """
+
+from threading import RLock
+
+lock = RLock()
+
+class InvokedFlag:
+    def __init__(self):
+        self.invoked = False
+
+def lock_function_once(f, flag):
+    def wrapper(*args, **kwargs):
+        if not flag.invoked:
+            with lock:
+                if not flag.invoked:
+                    result = f(*args, **kwargs)
+                    flag.invoked = True
+                    return result
+                return f(*args, **kwargs)
+        return f(*args, **kwargs)
+    return wrapper
+
+"""
+        if object_type == TempObjectType.TABLE_FUNCTION:
+            func_code = f"""{func_code}
+init_invoked = InvokedFlag()
+process_invoked = InvokedFlag()
+end_partition_invoked = InvokedFlag()
+
+class {_DEFAULT_HANDLER_NAME}(func):
+    def __init__(self):
+        lock_function_once(super().__init__, init_invoked)()
+
+    def process(self, {args}):
+        return lock_function_once(super().process, process_invoked)({args})
+"""
+            if hasattr(func, "end_partition"):
+                func_code = f"""{func_code}
+    def end_partition(self):
+        return lock_function_once(super().end_partition, end_partition_invoked)()
+"""
+        elif is_pandas_udf:
+            pandas_code = f"""
 import pandas
 
 {_DEFAULT_HANDLER_NAME}._sf_vectorized_input = pandas.DataFrame
 """.rstrip()
-        if max_batch_size:
-            pandas_code = f"""
+            if max_batch_size:
+                pandas_code = f"""
 {pandas_code}
 {_DEFAULT_HANDLER_NAME}._sf_max_batch_size = {int(max_batch_size)}
 """.rstrip()
-        if is_dataframe_input:
-            func_code = f"""
+            if is_dataframe_input:
+                func_code = f"""{func_code}
+invoked = InvokedFlag()
+
 def {_DEFAULT_HANDLER_NAME}(df):
-    return func(df)
+    return lock_function_once(func, invoked)(df)
 """.rstrip()
-        else:
-            func_code = f"""
+            else:
+                func_code = f"""{func_code}
+invoked = InvokedFlag()
+
 def {_DEFAULT_HANDLER_NAME}(df):
-    return func(*[df[idx] for idx in range(df.shape[1])])
+    return lock_function_once(func, invoked)(*[df[idx] for idx in range(df.shape[1])])
 """.rstrip()
-        func_code = f"""
+            func_code = f"""
 {func_code}
 {pandas_code}
 """.rstrip()
-    else:
-        func_code = f"""
+        else:
+            func_code = f"""{func_code}
+invoked = InvokedFlag()
+
 def {_DEFAULT_HANDLER_NAME}({args}):
-    return func({args})
+    return lock_function_once(func, invoked)({args})
 """.rstrip()
+
     return f"""
 {deserialization_code}
 {func_code}
@@ -400,6 +501,9 @@ def resolve_imports_and_packages(
     is_pandas_udf: bool = False,
     is_dataframe_input: bool = False,
     max_batch_size: Optional[int] = None,
+    *,
+    statement_params: Optional[Dict[str, str]] = None,
+    source_code_display: bool = False,
 ) -> Tuple[str, str, str, str, str]:
     upload_stage = (
         unwrap_stage_location_single_quote(stage_location)
@@ -423,14 +527,20 @@ def resolve_imports_and_packages(
                     "or a tuple of the file path (str) and the import path (str)."
                 )
             udf_level_imports[resolved_import_tuple[0]] = resolved_import_tuple[1:]
-        all_urls = session._resolve_imports(upload_stage, udf_level_imports)
+        all_urls = session._resolve_imports(
+            upload_stage, udf_level_imports, statement_params=statement_params
+        )
+    elif imports is None:
+        all_urls = session._resolve_imports(
+            upload_stage, statement_params=statement_params
+        )
     else:
-        all_urls = session._resolve_imports(upload_stage)
+        all_urls = []
 
     # resolve packages
     resolved_packages = (
         session._resolve_packages(packages, include_pandas=is_pandas_udf)
-        if packages
+        if packages is not None
         else session._resolve_packages(
             [], session._packages, validate_package=False, include_pandas=is_pandas_udf
         )
@@ -445,7 +555,13 @@ def resolve_imports_and_packages(
         udf_file_name_base = f"udf_py_{random_number()}"
         udf_file_name = f"{udf_file_name_base}.zip"
         code = generate_python_code(
-            func, arg_names, is_pandas_udf, is_dataframe_input, max_batch_size
+            func,
+            arg_names,
+            object_type,
+            is_pandas_udf,
+            is_dataframe_input,
+            max_batch_size,
+            source_code_display=source_code_display,
         )
         if len(code) > _MAX_INLINE_CLOSURE_SIZE_BYTES:
             dest_prefix = get_udf_upload_prefix(udf_name)
@@ -467,6 +583,7 @@ def resolve_imports_and_packages(
                     source_compression="DEFLATE",
                     compress_data=False,
                     overwrite=True,
+                    is_in_udf=True,
                 )
             all_urls.append(upload_file_stage_location)
             inline_code = None
@@ -521,7 +638,10 @@ def create_python_udf_or_sp(
     replace: bool,
     inline_python_code: Optional[str] = None,
 ) -> None:
-    return_sql_type = convert_sp_to_sf_type(return_type)
+    if isinstance(return_type, StructType):
+        return_sql = f'RETURNS TABLE ({",".join(f"{field.name} {convert_sp_to_sf_type(field.datatype)}" for field in return_type.fields)})'
+    else:
+        return_sql = f"RETURNS {convert_sp_to_sf_type(return_type)}"
     input_sql_types = [convert_sp_to_sf_type(arg.datatype) for arg in input_args]
     sql_func_args = ",".join(
         [f"{a.name} {t}" for a, t in zip(input_args, input_sql_types)]
@@ -541,7 +661,7 @@ $$
     create_query = f"""
 CREATE {"OR REPLACE " if replace else ""}
 {"TEMPORARY" if is_temporary else ""} {object_type.value} {object_name}({sql_func_args})
-RETURNS {return_sql_type}
+{return_sql}
 LANGUAGE PYTHON
 RUNTIME_VERSION=3.8
 {imports_in_sql}
