@@ -20,7 +20,7 @@ import zipfile
 from enum import Enum
 from json import JSONEncoder
 from random import choice
-from typing import IO, Any, Iterator, List, Optional, Type
+from typing import IO, Any, Dict, Iterator, List, Optional, Type
 
 import snowflake.snowpark
 from snowflake.connector.cursor import ResultMetadata, SnowflakeCursor
@@ -110,6 +110,14 @@ COPY_OPTIONS = {
     "LOAD_UNCERTAIN_FILES",
 }
 
+QUERY_TAG_STRING = "QUERY_TAG"
+SKIP_LEVELS_TWO = (
+    2  # limit traceback to return up to 2 stack trace entries from traceback object tb
+)
+SKIP_LEVELS_THREE = (
+    3  # limit traceback to return up to 3 stack trace entries from traceback object tb
+)
+
 
 class TempObjectType(Enum):
     TABLE = "TABLE"
@@ -371,6 +379,22 @@ def create_statement_query_tag(skip_levels: int = 0) -> str:
     return "".join(stack[:-skip_levels] if skip_levels else stack)
 
 
+def create_or_update_statement_params_with_query_tag(
+    statement_params: Optional[Dict[str, str]] = None,
+    exists_session_query_tag: Optional[str] = None,
+    skip_levels: int = 0,
+) -> Dict[str, str]:
+    if exists_session_query_tag or (
+        statement_params and QUERY_TAG_STRING in statement_params
+    ):
+        return statement_params
+
+    ret = statement_params or {}
+    # as create_statement_query_tag is called by the method, skip_levels needs to +1 to skip the current call
+    ret[QUERY_TAG_STRING] = create_statement_query_tag(skip_levels + 1)
+    return ret
+
+
 def get_stage_file_prefix_length(stage_location: str) -> int:
     normalized = unwrap_stage_location_single_quote(stage_location)
     if not normalized.endswith("/"):
