@@ -1,8 +1,8 @@
 #
 # Copyright (c) 2012-2022 Snowflake Computing Inc. All rights reserved.
 #
-
-from typing import Dict, Iterable, Optional, Union
+import warnings
+from typing import Dict, Iterable, List, Optional, Union
 
 import snowflake.snowpark  # for forward references of type hints
 from snowflake.snowpark._internal.analyzer.snowflake_plan_node import (
@@ -10,8 +10,11 @@ from snowflake.snowpark._internal.analyzer.snowflake_plan_node import (
     SaveMode,
     SnowflakeCreateTable,
 )
-from snowflake.snowpark._internal.telemetry import dfw_action_telemetry
-from snowflake.snowpark._internal.type_utils import ColumnOrName
+from snowflake.snowpark._internal.telemetry import (
+    add_api_call,
+    dfw_collect_api_telemetry,
+)
+from snowflake.snowpark._internal.type_utils import ColumnOrSqlExpr
 from snowflake.snowpark._internal.utils import (
     normalize_remote_file_or_dir,
     str_to_enum,
@@ -19,6 +22,7 @@ from snowflake.snowpark._internal.utils import (
 )
 from snowflake.snowpark.column import Column
 from snowflake.snowpark.functions import sql_expr
+from snowflake.snowpark.row import Row
 
 
 class DataFrameWriter:
@@ -27,21 +31,16 @@ class DataFrameWriter:
     To use this object:
 
     1. Create an instance of a :class:`DataFrameWriter` by accessing the :attr:`DataFrame.write` property.
-    2. Specify the save mode by calling :meth:`mode`, which returns the same
+    2. (Optional) Specify the save mode by calling :meth:`mode`, which returns the same
        :class:`DataFrameWriter` that is configured to save data using the specified mode.
        The default mode is "errorifexists".
-    3. Call the :meth:`save_as_table` method to save the data to the specified destination.
-
-    Example::
-
-        df.write.mode("overwrite").save_as_table("table1")
-
-
+    3. Call :meth:`save_as_table` or :meth:`copy_into_location` to save the data to the
+       specified destination.
     """
 
-    def __init__(self, dataframe: "snowflake.snowpark.dataframe.DataFrame"):
+    def __init__(self, dataframe: "snowflake.snowpark.dataframe.DataFrame") -> None:
         self._dataframe = dataframe
-        self._save_mode = SaveMode.APPEND  # spark default value is error.
+        self._save_mode = SaveMode.ERROR_IF_EXISTS
 
     def mode(self, save_mode: str) -> "DataFrameWriter":
         """Set the save mode of this :class:`DataFrameWriter`.
@@ -65,20 +64,24 @@ class DataFrameWriter:
         self._save_mode = str_to_enum(save_mode.lower(), SaveMode, "`save_mode`")
         return self
 
-    @dfw_action_telemetry
+    @dfw_collect_api_telemetry
     def save_as_table(
         self,
         table_name: Union[str, Iterable[str]],
         *,
         mode: Optional[str] = None,
+        column_order: str = "index",
         create_temp_table: bool = False,
+        table_type: str = "",
+        statement_params: Optional[Dict[str, str]] = None,
     ) -> None:
         """Writes the data to the specified table in a Snowflake database.
 
         Args:
             table_name: A string or list of strings that specify the table name or fully-qualified object identifier
                 (database name, schema name, and table name).
-            mode: One of the following values. When it's ``None``, the save mode set by calling ``df.write.mode(save_mode)`` is used.
+            mode: One of the following values. When it's ``None`` or not provided,
+                the save mode set by :meth:`mode` is used.
 
                 "append": Append data of this DataFrame to existing data.
 
@@ -88,16 +91,30 @@ class DataFrameWriter:
 
                 "ignore": Ignore this operation if data already exists.
 
-            create_temp_table: The to-be-created table will be temporary if this is set to ``True``. Default is ``False``.
+            column_order: When ``mode`` is "append", data will be inserted into the target table by matching column sequence or column name. Default is "index". When ``mode`` is not "append", the ``column_order`` makes no difference.
 
-        Example::
+                "index": Data will be inserted into the target table by column sequence.
+                "name": Data will be inserted into the target table by matching column names. If the target table has more columns than the source DataFrame, use this one.
 
-            df.write.mode("overwrite").save_as_table("table1")
-            df.write.save_as_table("table2", mode="overwrite", create_temp_table=True)
+            create_temp_table: (Deprecated) The to-be-created table will be temporary if this is set to ``True``.
+            table_type: The table type of table to be created. The supported values are: ``temp``, ``temporary``,
+                        and ``transient``. An empty string means to create a permanent table. Learn more about table
+                        types in https://docs.snowflake.com/en/user-guide/tables-temp-transient.html.
+            statement_params: Dictionary of statement level parameters to be set while executing this action.
 
+        Examples::
+
+            >>> df = session.create_dataframe([[1,2],[3,4]], schema=["a", "b"])
+            >>> df.write.mode("overwrite").save_as_table("my_table", table_type="temporary")
+            >>> session.table("my_table").collect()
+            [Row(A=1, B=2), Row(A=3, B=4)]
+            >>> df.write.save_as_table("my_table", mode="append", table_type="temporary")
+            >>> session.table("my_table").collect()
+            [Row(A=1, B=2), Row(A=3, B=4), Row(A=1, B=2), Row(A=3, B=4)]
+            >>> df.write.mode("overwrite").save_as_table("my_transient_table", table_type="transient")
+            >>> session.table("my_transient_table").collect()
+            [Row(A=1, B=2), Row(A=3, B=4)]
         """
-        # Snowpark scala doesn't have mode as a param but pyspark has it.
-        # They both have mode()
         save_mode = (
             str_to_enum(mode.lower(), SaveMode, "'mode'") if mode else self._save_mode
         )
@@ -105,34 +122,52 @@ class DataFrameWriter:
             table_name if isinstance(table_name, str) else ".".join(table_name)
         )
         validate_object_name(full_table_name)
+        if column_order is None or column_order.lower() not in ("name", "index"):
+            raise ValueError("'column_order' must be either 'name' or 'index'")
+        column_names = (
+            self._dataframe.columns if column_order.lower() == "name" else None
+        )
+
+        if create_temp_table:
+            warnings.warn(
+                "create_temp_table is deprecated. We still respect this parameter when it is True but "
+                'please consider using `table_type="temporary"` instead.',
+                DeprecationWarning,
+                # warnings.warn -> @dfw_collect_api_telemetry -> save_as_table
+                stacklevel=3,
+            )
+            table_type = "temporary"
+
+        if table_type and table_type.lower() not in ["temp", "temporary", "transient"]:
+            raise ValueError(
+                "Unsupported table type. Expected table types: temp/temporary, transient"
+            )
+
         create_table_logic_plan = SnowflakeCreateTable(
             full_table_name,
+            column_names,
             save_mode,
             self._dataframe._plan,
-            create_temp_table,
+            table_type,
         )
         session = self._dataframe._session
         snowflake_plan = session._analyzer.resolve(create_table_logic_plan)
-        session._conn.execute(snowflake_plan)
+        session._conn.execute(snowflake_plan, _statement_params=statement_params)
 
     def copy_into_location(
         self,
         location: str,
         *,
-        partition_by: Optional[ColumnOrName] = None,
+        partition_by: Optional[ColumnOrSqlExpr] = None,
         file_format_name: Optional[str] = None,
         file_format_type: Optional[str] = None,
         format_type_options: Optional[Dict[str, str]] = None,
         header: bool = False,
+        statement_params: Optional[Dict[str, str]] = None,
         **copy_options: Optional[str],
-    ) -> None:
+    ) -> List[Row]:
         """Executes a `COPY INTO <location> <https://docs.snowflake.com/en/sql-reference/sql/copy-into-location.html>`__ to unload data from a ``DataFrame`` into one or more files in a stage or external stage.
 
-        Example::
-
-            df = session.create_dataframe([["John", "Berry"], ["Rick", "Berry"], ["Anthony", "Davis"]], schema = ["FIRST_NAME", "LAST_NAME"])
-            df.write.copy_into_location("@my_stage_location", partition_by=col("LAST_NAME"), file_format_type="csv")
-
         Args:
             location: The destination stage location.
             partition_by: Specifies an expression used to partition the unloaded table rows into separate files. It can be a :class:`Column`, a column name, or a SQL expression.
@@ -140,18 +175,45 @@ class DataFrameWriter:
             file_format_type: Specifies the type of files unloaded from the table. If a format type is specified, additional format-specific options can be specified in ``format_type_options``.
             format_type_options: Depending on the ``file_format_type`` specified, you can include more format specific options. Use the options documented in the `Format Type Options <https://docs.snowflake.com/en/sql-reference/sql/copy-into-location.html#format-type-options-formattypeoptions>`__.
             header: Specifies whether to include the table column headings in the output files.
+            statement_params: Dictionary of statement level parameters to be set while executing this action.
             copy_options: The kwargs that are used to specify the copy options. Use the options documented in the `Copy Options <https://docs.snowflake.com/en/sql-reference/sql/copy-into-location.html#copy-options-copyoptions>`__.
+
+        Returns:
+            A list of :class:`Row` objects containing unloading results.
+
+        Example::
+
+            >>> # save this dataframe to a parquet file on the session stage
+            >>> df = session.create_dataframe([["John", "Berry"], ["Rick", "Berry"], ["Anthony", "Davis"]], schema = ["FIRST_NAME", "LAST_NAME"])
+            >>> remote_file_path = f"{session.get_session_stage()}/names.parquet"
+            >>> copy_result = df.write.copy_into_location(remote_file_path, file_format_type="parquet", header=True, overwrite=True, single=True)
+            >>> copy_result[0].rows_unloaded
+            3
+            >>> # the following code snippet just verifies the file content and is actually irrelevant to Snowpark
+            >>> # download this file and read it using pyarrow
+            >>> import os
+            >>> import tempfile
+            >>> import pyarrow.parquet as pq
+            >>> with tempfile.TemporaryDirectory() as tmpdirname:
+            ...     _ = session.file.get(remote_file_path, tmpdirname)
+            ...     pq.read_table(os.path.join(tmpdirname, "names.parquet"))
+            pyarrow.Table
+            FIRST_NAME: string not null
+            LAST_NAME: string not null
+            ----
+            FIRST_NAME: [["John","Rick","Anthony"]]
+            LAST_NAME: [["Berry","Berry","Davis"]]
         """
         stage_location = normalize_remote_file_or_dir(location)
         if isinstance(partition_by, str):
-            partition_by = sql_expr(partition_by).expression
+            partition_by = sql_expr(partition_by)._expression
         elif isinstance(partition_by, Column):
-            partition_by = partition_by.expression
+            partition_by = partition_by._expression
         elif partition_by is not None:
             raise TypeError(
                 f"'partition_by' is expected to be a column name, a Column object, or a sql expression. Got type {type(partition_by)}"
             )
-        return self._dataframe._with_plan(
+        df = self._dataframe._with_plan(
             CopyIntoLocationNode(
                 self._dataframe._plan,
                 stage_location,
@@ -162,6 +224,8 @@ class DataFrameWriter:
                 copy_options=copy_options,
                 header=header,
             )
-        )._internal_collect_with_tag()
+        )
+        add_api_call(df, "DataFrameWriter.copy_into_location")
+        return df._internal_collect_with_tag(statement_params=statement_params)
 
     saveAsTable = save_as_table
