@@ -25,7 +25,7 @@ from snowflake.snowpark._internal.analyzer.analyzer_utils import (
     escape_quotes,
     quote_name,
 )
-from snowflake.snowpark._internal.analyzer.datatype_mapper import to_sql
+from snowflake.snowpark._internal.analyzer.datatype_mapper import str_to_sql, to_sql
 from snowflake.snowpark._internal.analyzer.expression import Attribute
 from snowflake.snowpark._internal.analyzer.snowflake_plan import SnowflakePlanBuilder
 from snowflake.snowpark._internal.analyzer.snowflake_plan_node import (
@@ -38,6 +38,7 @@ from snowflake.snowpark._internal.analyzer.table_function import (
 )
 from snowflake.snowpark._internal.error_message import SnowparkClientExceptionMessages
 from snowflake.snowpark._internal.server_connection import ServerConnection
+from snowflake.snowpark._internal.telemetry import TelemetryField, set_api_call_source
 from snowflake.snowpark._internal.type_utils import (
     ColumnOrName,
     infer_schema,
@@ -65,6 +66,7 @@ from snowflake.snowpark._internal.utils import (
     validate_object_name,
     zip_file_or_directory_to_stream,
 )
+from snowflake.snowpark.column import Column
 from snowflake.snowpark.dataframe import DataFrame
 from snowflake.snowpark.dataframe_reader import DataFrameReader
 from snowflake.snowpark.file_operation import FileOperation
@@ -457,10 +459,14 @@ class Session:
         udf_level_import_paths: Optional[
             Dict[str, Tuple[Optional[str], Optional[str]]]
         ] = None,
+        *,
+        statement_params: Optional[Dict[str, str]] = None,
     ) -> List[str]:
         """Resolve the imports and upload local files (if any) to the stage."""
         resolved_stage_files = []
-        stage_file_list = self._list_files_in_stage(stage_location)
+        stage_file_list = self._list_files_in_stage(
+            stage_location, statement_params=statement_params
+        )
         normalized_stage_location = unwrap_stage_location_single_quote(stage_location)
 
         import_paths = udf_level_import_paths or self._import_paths
@@ -493,6 +499,7 @@ class Session:
                                 source_compression="DEFLATE",
                                 compress_data=False,
                                 overwrite=True,
+                                is_in_udf=True,
                             )
                     # local file
                     else:
@@ -511,14 +518,21 @@ class Session:
 
         return resolved_stage_files
 
-    def _list_files_in_stage(self, stage_location: Optional[str] = None) -> Set[str]:
+    def _list_files_in_stage(
+        self,
+        stage_location: Optional[str] = None,
+        *,
+        statement_params: Optional[Dict[str, str]] = None,
+    ) -> Set[str]:
         normalized = normalize_remote_file_or_dir(
             unwrap_single_quote(stage_location)
             if stage_location
             else self._session_stage
         )
         file_list = (
-            self.sql(f"ls {normalized}").select('"name"')._internal_collect_with_tag()
+            self.sql(f"ls {normalized}")
+            .select('"name"')
+            ._internal_collect_with_tag(statement_params=statement_params)
         )
         prefix_length = get_stage_file_prefix_length(stage_location)
         return {str(row[0])[prefix_length:] for row in file_list}
@@ -539,7 +553,7 @@ class Session:
         Use this method to add packages for UDFs as installing packages using
         `conda <https://docs.conda.io/en/latest/>`_. You can also find examples in
         :class:`~snowflake.snowpark.udf.UDFRegistration`. See details of
-        `third-party Python packages in Snowflake <https://docs.snowflake.com/en/LIMITEDACCESS/udf-python-packages.html>`_.
+        `third-party Python packages in Snowflake <https://docs.snowflake.com/en/developer-guide/udf/python/udf-python-packages.html>`_.
 
         Args:
             packages: A `requirement specifier <https://packaging.python.org/en/latest/glossary/#term-Requirement-Specifier>`_,
@@ -585,7 +599,7 @@ class Session:
             ``packages`` argument in :func:`functions.udf` or
             :meth:`session.udf.register() <snowflake.snowpark.udf.UDFRegistration.register>`.
 
-            2. We recommend you to `setup the local environment with Anaconda <https://docs.snowflake.com/en/LIMITEDACCESS/udf-python-packages.html#local-development-and-testing>`_,
+            2. We recommend you to `setup the local environment with Anaconda <https://docs.snowflake.com/en/developer-guide/udf/python/udf-python-packages.html#local-development-and-testing>`_,
             to ensure the consistent experience of a UDF between your local environment
             and the Snowflake server.
         """
@@ -661,7 +675,7 @@ class Session:
             ``packages`` argument in :func:`functions.udf` or
             :meth:`session.udf.register() <snowflake.snowpark.udf.UDFRegistration.register>`.
 
-            2. We recommend you to `setup the local environment with Anaconda <https://docs.snowflake.com/en/LIMITEDACCESS/udf-python-packages.html#local-development-and-testing>`_,
+            2. We recommend you to `setup the local environment with Anaconda <https://docs.snowflake.com/en/developer-guide/udf/python/udf-python-packages.html#local-development-and-testing>`_,
             to ensure the consistent experience of a UDF between your local environment
             and the Snowflake server.
         """
@@ -696,25 +710,44 @@ class Session:
         )
         for package in packages:
             if isinstance(package, ModuleType):
-                package_name = MODULE_NAME_TO_PACKAGE_NAME_MAP.get(
+                package = MODULE_NAME_TO_PACKAGE_NAME_MAP.get(
                     package.__name__, package.__name__
                 )
-                package = f"{package_name}=={pkg_resources.get_distribution(package_name).version}"
+                package = (
+                    f"{package}=={pkg_resources.get_distribution(package).version}"
+                )
                 use_local_version = True
             else:
                 package = package.strip().lower()
                 use_local_version = False
             package_req = pkg_resources.Requirement.parse(package)
-            # get the standard package name
-            package_name = package_req.key
+            # get the standard package name if there is no underscore
+            # underscores are discouraged in package names, but are still used in Anaconda channel
+            # pkg_resources.Requirement.parse will convert all underscores to dashes
+            package_name = (
+                package if not use_local_version and "_" in package else package_req.key
+            )
             if validate_package:
                 if package_name not in valid_packages:
+                    is_anaconda_terms_acknowledged = self._run_query(
+                        "select system$are_anaconda_terms_acknowledged()"
+                    )[0][0]
+                    if is_anaconda_terms_acknowledged:
+                        detailed_err_msg = (
+                            "it is not available in Snowflake. Check information_schema.packages "
+                            "to see available packages for UDFs. If this package is a "
+                            '"pure-Python" package, you can find the directory of this package '
+                            "and add it via session.add_import(). See details at "
+                            "https://docs.snowflake.com/en/developer-guide/snowpark/python/creating-udfs.html#using-third-party-packages-from-anaconda-in-a-udf."
+                        )
+                    else:
+                        detailed_err_msg = (
+                            "Anaconda terms must be accepted by ORGADMIN to use "
+                            "Anaconda 3rd party packages. Please follow the instructions at "
+                            "https://docs.snowflake.com/en/developer-guide/udf/python/udf-python-packages.html#using-third-party-packages-from-anaconda."
+                        )
                     raise ValueError(
-                        f"Cannot add package {package_name} because it is not "
-                        f"available in Snowflake. Check information_schema.packages "
-                        f"to see available packages for UDFs. If this package is a "
-                        f'"pure-Python" package, you can find the directory of this package '
-                        f"and add it via session.add_import()."
+                        f"Cannot add package {package_name} because {detailed_err_msg}"
                     )
                 elif not use_local_version:
                     try:
@@ -792,7 +825,7 @@ class Session:
     @query_tag.setter
     def query_tag(self, tag: str) -> None:
         if tag:
-            self._conn.run_query(f"alter session set query_tag = '{tag}'")
+            self._conn.run_query(f"alter session set query_tag = {str_to_sql(tag)}")
         else:
             self._conn.run_query("alter session unset query_tag")
         self._query_tag = tag
@@ -808,7 +841,7 @@ class Session:
         Examples::
 
             >>> df1 = session.create_dataframe([[1, 2], [3, 4]], schema=["a", "b"])
-            >>> df1.write.save_as_table("my_table", mode="overwrite", create_temp_table=True)
+            >>> df1.write.save_as_table("my_table", mode="overwrite", table_type="temporary")
             >>> session.table("my_table").collect()
             [Row(A=1, B=2), Row(A=3, B=4)]
             >>> current_db = session.get_current_database()
@@ -820,7 +853,10 @@ class Session:
         if not isinstance(name, str) and isinstance(name, Iterable):
             name = ".".join(name)
         validate_object_name(name)
-        return Table(name, self)
+        t = Table(name, self)
+        # Replace API call origin for table
+        set_api_call_source(t, "Session.table")
+        return t
 
     def table_function(
         self,
@@ -874,10 +910,12 @@ class Session:
         func_expr = _create_table_function_expression(
             func_name, *func_arguments, **func_named_arguments
         )
-        return DataFrame(
+        d = DataFrame(
             self,
             TableFunctionRelation(func_expr),
         )
+        set_api_call_source(d, f"Session.table_function[{func_expr.func_name}]")
+        return d
 
     def sql(self, query: str) -> DataFrame:
         """
@@ -896,7 +934,12 @@ class Session:
             >>> df.collect()
             [Row(1/2=Decimal('0.500000'))]
         """
-        return DataFrame(self, self._plan_builder.query(query, None))
+        return DataFrame(
+            self,
+            self._plan_builder.query(
+                query, None, api_calls=[{TelemetryField.NAME.value: "Session.sql"}]
+            ),
+        )
 
     @property
     def read(self) -> "DataFrameReader":
@@ -944,6 +987,7 @@ class Session:
         quote_identifiers: bool = True,
         auto_create_table: bool = False,
         create_temp_table: bool = False,
+        overwrite: bool = False,
     ) -> Table:
         """Writes a pandas DataFrame to a table in Snowflake and returns a
         Snowpark :class:`DataFrame` object referring to the table where the
@@ -970,6 +1014,10 @@ class Session:
                 table, you can always create your own table before calling this function. For example, auto-created
                 tables will store :class:`list`, :class:`tuple` and :class:`dict` as strings in a VARCHAR column.
             create_temp_table: The to-be-created table will be temporary if this is set to ``True``.
+            overwrite: Default value is ``False`` and the Pandas DataFrame data is appended to the existing table. If set to ``True`` and if auto_create_table is also set to ``True``,
+                then it drops the table. If set to ``True`` and if auto_create_table is set to ``False``,
+                then it trunctates the table. Note that in both cases (when overwrite is set to ``True``) it will replace the existing
+                contents of the table with that of the passed in Pandas DataFrame.
 
         Example::
 
@@ -981,6 +1029,20 @@ class Session:
             0   1  Steve
             1   2    Bob
 
+            >>> pandas_df2 = pd.DataFrame([(3, "John")], columns=["id", "name"])
+            >>> snowpark_df2 = session.write_pandas(pandas_df2, "write_pandas_table", auto_create_table=False)
+            >>> snowpark_df2.to_pandas()
+               id   name
+            0   1  Steve
+            1   2    Bob
+            2   3   John
+
+            >>> pandas_df3 = pd.DataFrame([(1, "Jane")], columns=["id", "name"])
+            >>> snowpark_df3 = session.write_pandas(pandas_df3, "write_pandas_table", auto_create_table=False, overwrite=True)
+            >>> snowpark_df3.to_pandas()
+               id  name
+            0   1  Jane
+
         Note:
             Unless ``auto_create_table`` is ``True``, you must first create a table in
             Snowflake that the passed in pandas DataFrame can be written to. If
@@ -1014,6 +1076,7 @@ class Session:
                 quote_identifiers=quote_identifiers,
                 auto_create_table=auto_create_table,
                 create_temp_table=create_temp_table,
+                overwrite=overwrite,
             )
         except ProgrammingError as pe:
             if pe.msg.endswith("does not exist"):
@@ -1024,7 +1087,9 @@ class Session:
                 raise pe
 
         if success:
-            return self.table(location)
+            t = self.table(location)
+            set_api_call_source(t, "Session.write_pandas")
+            return t
         else:
             raise SnowparkClientExceptionMessages.DF_PANDAS_GENERAL_EXCEPTION(
                 str(ci_output)
@@ -1073,7 +1138,7 @@ class Session:
             [Row(A=1, B=2), Row(A=3, B=4)]
             >>> session.create_dataframe([Row(a=1, b=2, c=3, d=4)]).collect()
             [Row(A=1, B=2, C=3, D=4)]
-            >>> session.createDataFrame([{"a": 1}, {"b": 2}]).collect()
+            >>> session.create_dataframe([{"a": 1}, {"b": 2}]).collect()
             [Row(A=1, B=None), Row(A=None, B=2)]
 
             >>> # create a dataframe from a pandas Dataframe
@@ -1105,7 +1170,7 @@ class Session:
             sf_database = self._conn._get_current_parameter("database", quoted=False)
             sf_schema = self._conn._get_current_parameter("schema", quoted=False)
 
-            return self.write_pandas(
+            t = self.write_pandas(
                 data,
                 table_name,
                 database=sf_database,
@@ -1114,6 +1179,8 @@ class Session:
                 auto_create_table=True,
                 create_temp_table=True,
             )
+            set_api_call_source(t, "Session.create_dataframe[pandas]")
+            return t
 
         # infer the schema based on the data
         names = None
@@ -1134,14 +1201,16 @@ class Session:
             )
 
         def convert_row_to_list(
-            row: Union[Dict, List, Tuple], names: List[str]
+            row: Union[Iterable[Any], Any], names: List[str]
         ) -> List:
             row_dict = None
-            if not row:
+            if row is None:
                 row = [None]
             elif isinstance(row, (tuple, list)):
-                if getattr(row, "_fields", None):  # Row or namedtuple
-                    row_dict = row.asDict() if isinstance(row, Row) else row._asdict()
+                if not row:
+                    row = [None]
+                elif getattr(row, "_fields", None):  # Row or namedtuple
+                    row_dict = row.as_dict() if isinstance(row, Row) else row._asdict()
             elif isinstance(row, dict):
                 row_dict = row.copy()
             else:
@@ -1149,6 +1218,7 @@ class Session:
 
             if row_dict:
                 # fill None if the key doesn't exist
+                row_dict = {quote_name(k): v for k, v in row_dict.items()}
                 return [row_dict.get(name) for name in names]
             else:
                 # check the length of every row, which should be same across data
@@ -1162,16 +1232,14 @@ class Session:
                 return list(row)
 
         # always overwrite the column names if they are provided via schema
-        if names:
-            for i, name in enumerate(names):
-                new_schema.fields[i].name = name
-        else:
+        if not names:
             names = [f.name for f in new_schema.fields]
-        rows = [convert_row_to_list(row, names) for row in data]
+        quoted_names = [quote_name(name) for name in names]
+        rows = [convert_row_to_list(row, quoted_names) for row in data]
 
         # get attributes and data types
         attrs, data_types = [], []
-        for field in new_schema.fields:
+        for field, quoted_name in zip(new_schema.fields, quoted_names):
             sf_type = (
                 StringType()
                 if isinstance(
@@ -1188,7 +1256,7 @@ class Session:
                 )
                 else field.datatype
             )
-            attrs.append(Attribute(quote_name(field.name), sf_type, field.nullable))
+            attrs.append(Attribute(quoted_name, sf_type, field.nullable))
             data_types.append(field.datatype)
 
         # convert all variant/time/geography/array/map data to string
@@ -1234,41 +1302,36 @@ class Session:
 
         # construct a project statement to convert string value back to variant
         project_columns = []
-        for field in new_schema.fields:
+        for field, name in zip(new_schema.fields, names):
             if isinstance(field.datatype, DecimalType):
                 project_columns.append(
                     to_decimal(
-                        column(field.name),
+                        column(name),
                         field.datatype.precision,
                         field.datatype.scale,
-                    ).as_(field.name)
+                    ).as_(name)
                 )
             elif isinstance(field.datatype, TimestampType):
-                project_columns.append(to_timestamp(column(field.name)).as_(field.name))
+                project_columns.append(to_timestamp(column(name)).as_(name))
             elif isinstance(field.datatype, TimeType):
-                project_columns.append(to_time(column(field.name)).as_(field.name))
+                project_columns.append(to_time(column(name)).as_(name))
             elif isinstance(field.datatype, DateType):
-                project_columns.append(to_date(column(field.name)).as_(field.name))
+                project_columns.append(to_date(column(name)).as_(name))
             elif isinstance(field.datatype, VariantType):
-                project_columns.append(
-                    to_variant(parse_json(column(field.name))).as_(field.name)
-                )
+                project_columns.append(to_variant(parse_json(column(name))).as_(name))
             elif isinstance(field.datatype, GeographyType):
-                project_columns.append(to_geography(column(field.name)).as_(field.name))
+                project_columns.append(to_geography(column(name)).as_(name))
             elif isinstance(field.datatype, ArrayType):
-                project_columns.append(
-                    to_array(parse_json(column(field.name))).as_(field.name)
-                )
+                project_columns.append(to_array(parse_json(column(name))).as_(name))
             elif isinstance(field.datatype, MapType):
-                project_columns.append(
-                    to_object(parse_json(column(field.name))).as_(field.name)
-                )
+                project_columns.append(to_object(parse_json(column(name))).as_(name))
             else:
-                project_columns.append(column(field.name))
+                project_columns.append(column(name))
 
-        return DataFrame(self, SnowflakeValues(attrs, converted)).select(
-            project_columns
-        )
+        df = DataFrame(self, SnowflakeValues(attrs, converted)).select(project_columns)
+        # Get rid of the select statement api call here
+        set_api_call_source(df, "Session.create_dataframe[values]")
+        return df
 
     def range(self, start: int, end: Optional[int] = None, step: int = 1) -> DataFrame:
         """
@@ -1292,7 +1355,9 @@ class Session:
             [Row(ID=1), Row(ID=3), Row(ID=5), Row(ID=7), Row(ID=9)]
         """
         range_plan = Range(0, start, step) if end is None else Range(start, end, step)
-        return DataFrame(self, range_plan)
+        df = DataFrame(self, range_plan)
+        set_api_call_source(df, "Session.range")
+        return df
 
     def get_current_database(self) -> Optional[str]:
         """
@@ -1337,10 +1402,7 @@ class Session:
         Args:
             database: The database name.
         """
-        if database:
-            self._run_query(f"use database {database}")
-        else:
-            raise ValueError("'database' must not be empty or None.")
+        self._use_object(database, "database")
 
     def use_schema(self, schema: str) -> None:
         """Specifies the active/current schema for the session.
@@ -1348,10 +1410,7 @@ class Session:
         Args:
             schema: The schema name.
         """
-        if schema:
-            self._run_query(f"use schema {schema}")
-        else:
-            raise ValueError("'schema' must not be empty or None.")
+        self._use_object(schema, "schema")
 
     def use_warehouse(self, warehouse: str) -> None:
         """Specifies the active/current warehouse for the session.
@@ -1359,10 +1418,7 @@ class Session:
         Args:
             warehouse: the warehouse name.
         """
-        if warehouse:
-            self._run_query(f"use warehouse {warehouse}")
-        else:
-            raise ValueError("'warehouse' must not be empty or None.")
+        self._use_object(warehouse, "warehouse")
 
     def use_role(self, role: str) -> None:
         """Specifies the active/current primary role for the session.
@@ -1370,10 +1426,14 @@ class Session:
         Args:
             role: the role name.
         """
-        if role:
-            self._run_query(f"use role {role}")
+        self._use_object(role, "role")
+
+    def _use_object(self, object_name: str, object_type: str) -> None:
+        if object_name:
+            validate_object_name(object_name)
+            self._run_query(f"use {object_type} {object_name}")
         else:
-            raise ValueError("'role' must not be empty or None.")
+            raise ValueError(f"'{object_type}' must not be empty or None.")
 
     @property
     def telemetry_enabled(self) -> bool:
@@ -1423,8 +1483,8 @@ class Session:
     @property
     def udtf(self) -> UDTFRegistration:
         """
-        Returns a :class:`udf.UDTFRegistration` object that you can use to register UDFs.
-        See details of how to use this object in :class:`udf.UDFRegistration`.
+        Returns a :class:`udtf.UDTFRegistration` object that you can use to register UDTFs.
+        See details of how to use this object in :class:`udtf.UDTFRegistration`.
         """
         return self._udtf_registration
 
@@ -1465,8 +1525,13 @@ class Session:
 
         sql_args = []
         for arg in args:
-            sql_args.append(to_sql(arg, infer_type(arg)))
-        return self.sql(f"CALL {sproc_name}({', '.join(sql_args)})").collect()[0][0]
+            if isinstance(arg, Column):
+                sql_args.append(self._analyzer.analyze(arg._expression))
+            else:
+                sql_args.append(to_sql(arg, infer_type(arg)))
+        df = self.sql(f"CALL {sproc_name}({', '.join(sql_args)})")
+        set_api_call_source(df, f"Session.call[{sproc_name}]")
+        return df.collect()[0][0]
 
     @deprecate(
         deprecate_version="0.7.0",
@@ -1542,12 +1607,14 @@ class Session:
             raise ValueError("mode must be one of ('OBJECT', 'ARRAY', 'BOTH')")
         if isinstance(input, str):
             input = col(input)
-        return DataFrame(
+        df = DataFrame(
             self,
             TableFunctionRelation(
                 FlattenFunction(input._expression, path, outer, recursive, mode)
             ),
         )
+        set_api_call_source(df, "Session.flatten")
+        return df
 
     def query_history(self) -> QueryHistory:
         """Create an instance of :class:`QueryHistory` as a context manager to record queries that are pushed down to the Snowflake database.
@@ -1563,6 +1630,7 @@ class Session:
         return query_listener
 
     def _table_exists(self, table_name: str):
+        # TODO: Support qualified table names
         tables = self._run_query(f"show tables like '{table_name}'")
         return tables is not None and len(tables) > 0
 
